.include "cpctelera.h.s"

.include "systems/entity_manager.inc"
.include "systems/level_manager.inc"
.include "systems/physics_manager.inc"

.area _DATA
.area _CODE

;; cpctelera global declarations
.globl cpct_disableFirmware_asm
.globl cpct_setVideoMode_asm
.globl cpct_drawSprite_asm
.globl cpct_getScreenPtr_asm
.globl cpct_scanKeyboard_asm
.globl cpct_isAnyKeyPressed_asm
.globl cpct_isKeyPressed_asm
.globl cpct_setPalette_asm
.globl cpct_waitVSYNC_asm
.globl cpct_drawSolidBox_asm
.globl _spr_menu
.globl _spr_controls
.globl _game_palette

init_game:
    call cpct_disableFirmware_asm
    ld c, #0
    call cpct_setVideoMode_asm
    ld hl, #_game_palette
    ld de, #16
    call cpct_setPalette_asm

    ret

update_game:
    ld a, (entity_alive_count)
    cp #0
    ret z

    ld ix, #entity_array
    call physics_move_player
    cp #0
    ret z

    ld bc, #entity_size_bytes
    add ix, bc
    dec a
    ret z
    update_game_loop:
        push af
        ld a, entity_get_type(ix)
        cp #entity_type_enemy
        jr nz, update_gameloop_noenemy
        call physics_move_enemy

        update_gameloop_noenemy:
        pop af
        ld bc, #entity_size_bytes
        add ix, bc
        dec a
        jp nz, update_game_loop
    ret

render_game:
    ld a, (entity_alive_count)
    cp #0
    ret z

    call level_draw

    ld ix, #entity_array
    ld a, (entity_alive_count)
    render_game_loop:
        push af
        ld de, #0xC000
        ld c, entity_get_x(ix)
        ;; x = entity_x * entity_width_bytes = entity_x * 4 = entity_x * 2^2
        sla c
        sla c
        ld b, entity_get_y(ix)
        ;; y = entity_y * entity_height_bytes = entity_y * 8 = entity_y * 2^3
        sla b
        sla b
        sla b
        call cpct_getScreenPtr_asm
        ex de, hl
        ld b, #sprite_height_bytes
        ld c, #sprite_width_bytes
        ld h, entity_get_sprite_h(ix)
        ld l, entity_get_sprite_l(ix)
        call cpct_drawSprite_asm
        pop af

        ld bc, #entity_size_bytes
        add ix, bc
        dec a
        jp nz, render_game_loop
    ret


main_menu:
    ld de, #CPCT_VMEM_START_ASM
    ld a, #0
    ld c, #10
    ld b, #200
    call cpct_drawSolidBox_asm
    ld de, #CPCT_VMEM_START_ASM
    ld c, #10
    ld b, #0
    call cpct_getScreenPtr_asm
    ex de, hl
    ld hl, #_spr_menu
    ld c, #60
    ld b, #200
    call cpct_drawSprite_asm
    menu_loop:
    call cpct_waitVSYNC_asm
    call cpct_scanKeyboard_asm
    ld hl, #Key_1
    call cpct_isKeyPressed_asm
    jr z, menu_loop_check2
    ret
    menu_loop_check2:
    ld hl, #Key_2
    call cpct_isKeyPressed_asm
    jr z, menu_loop
    jp controls_menu
    ret

controls_menu:
    ld de, #CPCT_VMEM_START_ASM
    ld c, #10
    ld b, #0
    call cpct_getScreenPtr_asm
    ex de, hl
    ld hl, #_spr_controls
    ld c, #60
    ld b, #200
    call cpct_drawSprite_asm
    controls_loop:
    call cpct_waitVSYNC_asm
    call cpct_scanKeyboard_asm
    ld hl, #Key_1
    call cpct_isKeyPressed_asm
    jr z, controls_loop_check2
    ld hl, #Key_W
    ld (physics_key_up), hl
    ld hl, #Key_S
    ld (physics_key_down), hl
    ld hl, #Key_A
    ld (physics_key_left), hl
    ld hl, #Key_D
    ld (physics_key_right),hl
    jp main_menu
    controls_loop_check2:
    ld hl, #Key_2
    call cpct_isKeyPressed_asm
    jr z, controls_loop_check3
    ld hl, #Key_Q
    ld (physics_key_up), hl
    ld hl, #Key_A
    ld (physics_key_down), hl
    ld hl, #Key_O
    ld (physics_key_left), hl
    ld hl, #Key_P
    ld (physics_key_right),hl
    jp main_menu
    controls_loop_check3:
    ld hl, #Key_3
    call cpct_isKeyPressed_asm
    jr z, controls_loop_check4
    ld hl, #Key_K
    ld (physics_key_up), hl
    ld hl, #Key_J
    ld (physics_key_down), hl
    ld hl, #Key_H
    ld (physics_key_left), hl
    ld hl, #Key_L
    ld (physics_key_right),hl
    jp main_menu
    controls_loop_check4:
    ld hl, #Key_4
    call cpct_isKeyPressed_asm
    jr z, controls_loop
    ld hl, #Joy0_Up
    ld (physics_key_up), hl
    ld hl, #Joy0_Down
    ld (physics_key_down), hl
    ld hl, #Joy0_Left
    ld (physics_key_left), hl
    ld hl, #Joy0_Right
    ld (physics_key_right),hl
    jp main_menu
    ret

frame_counter_start = 120
frame_counter: .db frame_counter_start

_main::
    call init_game
    cpctm_setBorder_asm #HW_BLACK
    call main_menu
    ld de, #CPCT_VMEM_START_ASM
    ld a, #0
    ld c, #64
    ld b, #200
    call cpct_drawSolidBox_asm
    call level_manager_init
    call entity_clear_entities

    call level_manager_init
    call render_game

loop:
    call cpct_waitVSYNC_asm

    ld a, (frame_counter)
    dec a
    ld (frame_counter), a
    jr nz, loop

    ld a, #frame_counter_start
    ld (frame_counter), a
    call cpct_scanKeyboard_asm
    call cpct_isAnyKeyPressed_asm
    jr z, loop

    call update_game
    call render_game
    jr loop
