.module entity_manager

.include "entity_manager.inc"
.include "level_manager.inc"

.globl _spr_bat
.globl _main

entity_zero:
    .db 0x00   ; Entity type (enemy, player, key, etc...)
    .db 0x00   ; X position in tiles
    .db 0x00   ; Y position in tiles
    .db 0x00   ; Life (max 200)
    .db 0x00   ; Attack power (dmg to make when attacking)
    .dw 0x0000 ; Sprite address
entity_size_bytes = . - entity_zero
entity_door:
    .db entity_type_door
    .db 0x08
    .db 0x08
    .db 0x00
    .db 0x00
    .dw _spr_bat

entity_alive_count: .db #0

entity_array: .ds entity_size_bytes*entity_max_count
entity_alive_end: .dw #entity_array

entity_create:
    ld a, (entity_alive_count)
    cp #entity_max_count
    ld a, #1
    ret z

    ld de, (entity_alive_end)
    ld bc, #entity_size_bytes
    ldir
    ld (entity_alive_end), de

    ld a, (entity_alive_count)
    inc a
    ld (entity_alive_count), a

    ld a, #0
    ret

entity_destroy:
    ld a, (entity_alive_count)
    cp #0
    ld a, #1
    ret z

    ex de, hl
    ld hl, (entity_alive_end)
    ld bc, #entity_size_bytes
    sbc hl, bc
    ld (entity_alive_end), hl
    ldir

    ld a, (entity_alive_count)
    dec a
    ld (entity_alive_count), a

    ld a, #0
    ret

entity_player_collision:
    push bc
    ld iy, #entity_array
    ld bc, #entity_size_bytes
    add iy, bc
    ld a, (entity_alive_count)
    dec a
    pop bc
    entity_player_collision_loop:
        push bc
        push af
        ld a, entity_get_y(iy)
        cp b
        jr nz, entity_player_collision_no_collision
        ld a, entity_get_x(iy)
        cp c
        jr nz, entity_player_collision_no_collision

        ld a, entity_get_type(iy)
        cp #entity_type_key
        jr nz, entity_player_collision_gem
        pop af
        pop bc
        call level_change
        ld a, #0
        ret
        entity_player_collision_gem:
        cp #entity_type_gem
        jr nz, #entity_player_collision_enemy
        jr entity_player_collision_no_collision
        entity_player_collision_enemy:
        cp #entity_type_enemy
        jr nz, #entity_player_collision_no_collision
        ld sp, #0xFFFF
        jp _main
        entity_player_collision_no_collision:
        pop af
        ld bc, #entity_size_bytes
        add iy, bc
        pop bc
        dec a
        jr nz, entity_player_collision_loop
    ld a, #1
    ret

entity_clear_entities:
    ld a, #0
    ld (entity_alive_count), a
    ld hl, #entity_array
    ld (entity_alive_end), hl
    ret
