.module level_manager

.include "level_manager.inc"
.include "entity_manager.inc"
.include "cpctelera.h.s"

.globl _tiles_0
.globl _tilemap1
.globl _tilemap2
.globl _tilemap3
.globl _game_palette
.globl _spr_player
.globl _spr_bat
.globl _spr_gem
.globl _spr_key
.globl _main

.globl cpct_etm_setDrawTilemap4x8_ag_asm
.globl cpct_etm_drawTilemap4x8_ag_asm
.globl cpct_setPALColour_asm
.globl cpct_setPalette_asm

current_level: .db 1
current_level_data: .dw level_forest
current_level_tilemap: .dw 0x00

level_manager_init:
    ld hl, #_game_palette
    ld de, #16
    call cpct_setPalette_asm

    ld c, #tilemap_width
    ld b, #tilemap_height
    ld de, #tilemap_width
    ld hl, #_tiles_0
    call cpct_etm_setDrawTilemap4x8_ag_asm

    cpctm_setBorder_asm #HW_GREEN
    ld a, #1
    ld (current_level), a
    ld hl, #level_forest
    ld (current_level_data), hl
    call level_load
    ret

level_load:
    ld hl, (current_level_data)
    ld de, #current_level_tilemap
    ld a, (hl)
    ld (de), a
    inc hl
    inc de
    ld a, (hl)
    ld (de), a
    inc hl
    ld h, (hl)
    ld l, #0
    call cpct_setPALColour_asm

    ld hl, (current_level_data)
    ld bc, #3
    add hl, bc
    ld a, (hl)
    inc hl
    ld de, #entity_array
    ld (entity_alive_count), a
    create_entity_loop:
        ld bc, #entity_size_bytes
        ldir
        dec a
        jr nz, create_entity_loop

    ret

level_change:
    ld a, (current_level)
    cp #1
    jr nz, level_load_next_check2
    cpctm_setBorder_asm #HW_YELLOW
    ld hl, #level_cave
    ld (current_level_data), hl
    ld a, #2
    ld (current_level), a
    call level_load
    jr level_load_next_check3
    level_load_next_check2:
    cp #2
    jr nz, level_load_next_check3
    cpctm_setBorder_asm #HW_RED
    ld hl, #level_lavacave
    ld (current_level_data), hl
    ld a, #3
    ld (current_level), a
    call level_load
    level_load_next_check3:
    cp #3
    jr nz, level_load_end
    ld sp, #0xFFFF
    jp _main
    level_load_end:
    ret

level_draw:
    ld hl, #0xC000
    ld de, (current_level_tilemap)
    call cpct_etm_drawTilemap4x8_ag_asm
    ret

level_check_collision:
    ld hl, (current_level_tilemap)
    ld a, b
    sla a
    sla a
    sla a
    sla a
    add a, c
    ld b, #0
    ld c, a
    add hl, bc
    ld a, (hl)
    sub #6
    and #0x80
    jr z, #level_check_collision_true
    ld a, #1
    ret
    level_check_collision_true:
    ld a, #0
    ret

level_forest:
    .dw _tilemap1
    .db 0x56
    .db 5
    .db entity_type_player ; Entity type (enemy, player, key, etc...)
    .db 0x07               ; X position in tiles
    .db 0x08               ; Y position in tiles
    .db 0x14               ; Life (max 255)
    .db 0x14               ; Attack power (dmg to make when attacking)
    .dw _spr_player        ; Sprite address

    .db entity_type_enemy  ; Entity type (enemy, player, key, etc...)
    .db 0x01               ; X position in tiles
    .db 0x08               ; Y position in tiles
    .db 0x14               ; Life (max 255)
    .db 0x14               ; Attack power (dmg to make when attacking)
    .dw _spr_bat           ; Sprite address

    .db entity_type_enemy  ; Entity type (enemy, player, key, etc...)
    .db 0x0E               ; X position in tiles
    .db 0x08               ; Y position in tiles
    .db 0x14               ; Life (max 255)
    .db 0x14               ; Attack power (dmg to make when attacking)
    .dw _spr_bat           ; Sprite address

    .db entity_type_gem  ; Entity type (enemy, player, key, etc...)
    .db 0x09               ; X position in tiles
    .db 0x01               ; Y position in tiles
    .db 0x00               ; Life (max 255)
    .db 0x00               ; Attack power (dmg to make when attacking)
    .dw _spr_gem           ; Sprite address

    .db entity_type_key  ; Entity type (enemy, player, key, etc...)
    .db 0x04               ; X position in tiles
    .db 0x0E               ; Y position in tiles
    .db 0x00               ; Life (max 255)
    .db 0x00               ; Attack power (dmg to make when attacking)
    .dw _spr_key           ; Sprite address

level_cave:
    .dw _tilemap3
    .db 0x5E
    .db 4
    .db entity_type_player ; Entity type (enemy, player, key, etc...)
    .db 0x0E               ; X position in tiles
    .db 0x0E               ; Y position in tiles
    .db 0x14               ; Life (max 255)
    .db 0x14               ; Attack power (dmg to make when attacking)
    .dw _spr_player        ; Sprite address

    .db entity_type_enemy  ; Entity type (enemy, player, key, etc...)
    .db 0x01               ; X position in tiles
    .db 0x01               ; Y position in tiles
    .db 0x14               ; Life (max 255)
    .db 0x14               ; Attack power (dmg to make when attacking)
    .dw _spr_bat           ; Sprite address

    .db entity_type_gem  ; Entity type (enemy, player, key, etc...)
    .db 0x08               ; X position in tiles
    .db 0x0E               ; Y position in tiles
    .db 0x00               ; Life (max 255)
    .db 0x00               ; Attack power (dmg to make when attacking)
    .dw _spr_gem           ; Sprite address

    .db entity_type_key  ; Entity type (enemy, player, key, etc...)
    .db 0x08               ; X position in tiles
    .db 0x05               ; Y position in tiles
    .db 0x00               ; Life (max 255)
    .db 0x00               ; Attack power (dmg to make when attacking)
    .dw _spr_key           ; Sprite address

level_lavacave:
    .dw _tilemap2
    .db 0x5C
    .db 4
    .db entity_type_player ; Entity type (enemy, player, key, etc...)
    .db 0x0E               ; X position in tiles
    .db 0x01               ; Y position in tiles
    .db 0x14               ; Life (max 255)
    .db 0x14               ; Attack power (dmg to make when attacking)
    .dw _spr_player        ; Sprite address

    .db entity_type_enemy  ; Entity type (enemy, player, key, etc...)
    .db 0x02               ; X position in tiles
    .db 0x0A               ; Y position in tiles
    .db 0x14               ; Life (max 255)
    .db 0x14               ; Attack power (dmg to make when attacking)
    .dw _spr_bat           ; Sprite address

    .db entity_type_gem  ; Entity type (enemy, player, key, etc...)
    .db 0x0E               ; X position in tiles
    .db 0x08               ; Y position in tiles
    .db 0x00               ; Life (max 255)
    .db 0x00               ; Attack power (dmg to make when attacking)
    .dw _spr_gem           ; Sprite address

    .db entity_type_key  ; Entity type (enemy, player, key, etc...)
    .db 0x01               ; X position in tiles
    .db 0x09               ; Y position in tiles
    .db 0x00               ; Life (max 255)
    .db 0x00               ; Attack power (dmg to make when attacking)
    .dw _spr_key           ; Sprite address

