.module physics_manager

.include "cpctelera.h.s"
.include "entity_manager.inc"
.include "physics_manager.inc"
.include "level_manager.inc"

.globl cpct_isKeyPressed_asm
.globl _main

physics_key_up: .dw Key_W
physics_key_down: .dw Key_S
physics_key_left: .dw Key_A
physics_key_right: .dw Key_D

physics_move_player:
    ld hl, (physics_key_up)
    call cpct_isKeyPressed_asm
    jp z, check_player_input_down
    call physics_move_up
    jp check_player_input_end
    check_player_input_down:
    ld hl, (physics_key_down)
    call cpct_isKeyPressed_asm
    jp z, check_player_input_left
    call physics_move_down
    jp check_player_input_end
    check_player_input_left:
    ld hl, (physics_key_left)
    call cpct_isKeyPressed_asm
    jp z, check_player_input_right
    call physics_move_left
    jp check_player_input_end
    check_player_input_right:
    ld hl, (physics_key_right)
    call cpct_isKeyPressed_asm
    jp z, check_player_input_end
    call physics_move_right

    check_player_input_end:
    ret

physics_move_enemy:
    ld iy, #entity_array
    ld b, entity_get_y(ix)
    ld c, entity_get_x(ix)
    ld a, entity_get_y(iy)
    sub b
    jr z, physics_move_enemy_horizontal
    cp #1
    jr z, physics_move_enemy_horizontal
    cp #-1
    jr z, physics_move_enemy_horizontal
    and #0x80
    jr z, physics_move_enemy_down
    push bc
    call physics_move_up
    pop bc
    jr physics_move_enemy_horizontal
    physics_move_enemy_down:
    push bc
    call physics_move_down
    pop bc
    physics_move_enemy_horizontal:
    ld a, entity_get_x(iy)
    sub c
    jr z, physics_move_enemy_end
    cp #1
    jr z, physics_move_enemy_end
    cp #-1
    jr z, physics_move_enemy_end
    and #0x80
    jr z, physics_move_enemy_right
    push bc
    call physics_move_left
    pop bc
    ret
    physics_move_enemy_right:
    push bc
    call physics_move_right
    pop bc
    ret

    physics_move_enemy_end:
    ld a, entity_get_y(iy)
    sub b
    cp #0
    jr z, move_enemy_end2
    cp #-1
    jr z, move_enemy_end2
    cp #1
    jr z, move_enemy_end2
    ret
    move_enemy_end2:
    ld a, entity_get_x(iy)
    sub c
    cp #0
    jr z, move_enemy_kill
    cp #-1
    jr z, move_enemy_kill
    cp #1
    jr z, move_enemy_kill
    ret
    move_enemy_kill:
    ld sp, #0xFFFF
    jp _main

physics_move_up:
    ld a, entity_get_y(ix)
    cp #0
    ret z

    ld b, a
    dec b
    ld c, entity_get_x(ix)
    call level_check_collision
    cp #0
    ret z

    ld b, entity_get_y(ix)
    dec b
    ld c, entity_get_x(ix)
    ld a, entity_get_type(ix)
    cp #0
    jr nz, physics_key_up_noplayer
    push af
    call entity_player_collision
    cp #0
    jr nz, entity_player_collision_up_continue
    pop af
    ld a, #0
    ret
    entity_player_collision_up_continue:
    pop af
    physics_key_up_noplayer:
    ld a, entity_get_y(ix)
    dec a
    ld entity_get_y(ix), a
    ret

physics_move_down:
    ld a, entity_get_y(ix)
    cp #tilemap_height-1
    ret z

    ld b, a
    inc b
    ld c, entity_get_x(ix)
    call level_check_collision
    cp #0
    ret z

    ld b, entity_get_y(ix)
    inc b
    ld c, entity_get_x(ix)
    ld a, entity_get_type(ix)
    cp #0
    jr nz, physics_key_down_noplayer
    push af
    call entity_player_collision
    cp #0
    jr nz, entity_player_collision_down_continue
    pop af
    ld a, #0
    ret
    entity_player_collision_down_continue:
    pop af
    physics_key_down_noplayer:
    ld a, entity_get_y(ix)
    inc a
    ld entity_get_y(ix), a
    ret

physics_move_left:
    ld a, entity_get_x(ix)
    cp #0
    ret z

    ld b, entity_get_y(ix)
    ld c, a
    dec c
    call level_check_collision
    cp #0
    ret z

    ld b, entity_get_y(ix)
    ld c, entity_get_x(ix)
    dec c
    ld a, entity_get_type(ix)
    cp #0
    jr nz, physics_key_left_noplayer
    push af
    call entity_player_collision
    cp #0
    jr nz, entity_player_collision_left_continue
    pop af
    ld a, #0
    ret
    entity_player_collision_left_continue:
    pop af
    physics_key_left_noplayer:
    ld a, entity_get_x(ix)
    dec a
    ld entity_get_x(ix), a
    ret

physics_move_right:
    ld a, entity_get_x(ix)
    cp #tilemap_width-1
    ret z

    ld b, entity_get_y(ix)
    ld c, a
    inc c
    call level_check_collision
    cp #0
    ret z

    ld b, entity_get_y(ix)
    ld c, entity_get_x(ix)
    inc c
    ld a, entity_get_type(ix)
    cp #0
    jr nz, physics_key_right_noplayer
    push af
    call entity_player_collision
    cp #0
    jr nz, entity_player_collision_right_continue
    pop af
    ld a, #0
    ret
    entity_player_collision_right_continue:
    pop af
    physics_key_right_noplayer:
    ld a, entity_get_x(ix)
    inc a
    ld entity_get_x(ix), a
    ret

